#!/bin/bash
################################################################################
# Modification Log (most recent first)                                         #
# ====================================                                         #
# Date        Author          Description                                      #
# ====        ======          ===========                                      #
# 20180920    Simon Vollett   Infra 641423: Initial code.                      #
#                                                                              #
# Description: This script reads a jobfile as specified by parameter and       #
#              executes sftp commands based on the contents of jobfile.        #
#                                                                              #
# Exit Codes:                                                                  #
# ===========                                                                  #
# 0: Script completed succesfully                                              #
# 1: Incorrect number of args                                                  #
# 2: Jobfile is not a readable file                                            #
# 3: SFTP attempt failed                                                       #
# 9: Unknown error                                                             #
################################################################################
function usage() { # echos a usage message
  echo "Usage:  $THIS_SCRIPT [jobfile]"
  echo "Params: 1) jobfile: Name of jobfile that contains individual instructions for sftp batch job."
  return 0
}

function now() { # echos date and time now
  DATENOW=$(date +"%Y-%m-%d")
  TIMENOW=$(date +"%T")
  echo "$DATENOW $TIMENOW"
  return 0
}

function log() { # Log debug information
  if [ $1 -le $DEBUG_LEVEL -a $DEBUG_LEVEL -ne 0 ]; then
    echo "$(now): $THIS_SCRIPT: $2" | tee -a $LOGFILE
  fi
return 0
}

################################################################################
# Initialise
DEBUG_LEVEL=2     # 0|1|2|3 -> None|Error|Info|All
LOGFILE="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/$(basename "${BASH_SOURCE[0]%.sh}.log")"
THIS_SCRIPT="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/$(basename "${BASH_SOURCE[0]}")";
THIS_SCRIPT_PATH="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)";
log 1 "################################################################################"
log 1 "# Initialise $THIS_SCRIPT"
log 2 "DEBUG_LEVEL: $DEBUG_LEVEL"
log 2 "LOGFILE: $LOGFILE"
log 2 "THIS_SCRIPT: $THIS_SCRIPT"
log 2 "THIS_SCRIPT_PATH: $THIS_SCRIPT_PATH"

if [ $# -ne 1 ]; then
  usage; log 1 "Error: Incorrect number of args. Exiting.";
  log 1 "################################################################################"
  exit 1
else
  log 3 "Correct number of args supplied, continue..."
fi

JOBFILE="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/sftp_batch/$1"
if [[ -r $JOBFILE ]]; then
  log 3 "JOBFILE: $JOBFILE is a readable file, continue..."
else
  log 1 "Error: JOBFILE: $JOBFILE is not a readable file. Exiting.";
  log 1 "################################################################################"
  exit 2
fi

log 2 "Reading $JOBFILE..."
LINE_NUM=0
while IFS="," read -r DEFAULT_NUMBER_OF_RETRIES TIME_BETWEEN_RETRIES REMOTE_HOST REMOTE_USER REMOTE_DIR FILE_PATTERN LOCAL_DIR MODE; do
  let "LINE_NUM=LINE_NUM+1"
  log 2 "Processing $JOBFILE: Line number $LINE_NUM"
  log 3 "DEFAULT_NUMBER_OF_RETRIES: $DEFAULT_NUMBER_OF_RETRIES read from $JOBFILE"
  log 3 "TIME_BETWEEN_RETRIES     : $TIME_BETWEEN_RETRIES      read from $JOBFILE"
  log 3 "REMOTE_HOST              : $REMOTE_HOST               read from $JOBFILE"
  log 3 "REMOTE_USER              : $REMOTE_USER               read from $JOBFILE"
  log 3 "REMOTE_DIR               : $REMOTE_DIR                read from $JOBFILE"
  log 3 "FILE_PATTERN             : $FILE_PATTERN              read from $JOBFILE"
  log 3 "LOCAL_DIR                : $LOCAL_DIR                 read from $JOBFILE"
  log 3 "MODE                     : $MODE                      read from $JOBFILE"
  log 2 "Executing: ./sftp_files.sh -n $DEFAULT_NUMBER_OF_RETRIES -b $TIME_BETWEEN_RETRIES -h $REMOTE_HOST -u $REMOTE_USER -d $REMOTE_DIR -f $FILE_PATTERN -D $LOCAL_DIR -m $MODE"
  cd $THIS_SCRIPT_PATH
  ./sftp_files.sh -n $DEFAULT_NUMBER_OF_RETRIES -b $TIME_BETWEEN_RETRIES -h $REMOTE_HOST -u $REMOTE_USER -d $REMOTE_DIR -f $FILE_PATTERN -D $LOCAL_DIR -m $MODE
  if [ $? -ne 0 ]; then
    log 1 "Error: SFTP attempt failed, exiting.";
    log 1 "################################################################################"
    exit 3
  else
    log 2 "$JOBFILE: Line number $LINE_NUM processed successfully"
  fi

done < $JOBFILE

log 1 "# Script $THIS_SCRIPT: Finished successfully"
log 1 "################################################################################"
exit 0
