#!/bin/bash
################################################################################
# Modification Log (most recent first)                                         #
# ====================================                                         #
# Date        Author          Description                                      #
# ====        ======          ===========                                      #
# 20180920    Simon Vollett   Infra 641423: Initial code.                      #
#                                                                              #
# Description: This script sftp's file/s between this and a remote host.       #
#                                                                              #
# Exit Codes:                                                                  #
# ===========                                                                  #
# 0: Script completed succesfully                                              #
# 1: Incorrect number of arguments                                             #
# 2: Invalid option supplied                                                   #
# 3: Mode parameter invalid                                                    #
# 4: SFTP attempt failed                                                       #
# 9: Unknown error                                                             #
################################################################################
function usage() { # echos a usage message
  echo "Usage:  /opt/prod/scripts/sftp_files.sh [-n number_of_retries] [-o connection_timeout] [-b time_between_retries] [-h remote_host] [-u remote_user] [-d remote_dir] [-f file_pattern] [-m mode]"
  echo "Param: -n default_number_of_retries -> Number of retries before sftp attempt should be aborted"
  echo "Param: -b time_between_retries      -> Seconds to wait before attempting sftp again after a failure"
  echo "Param: -h remote_host               -> Host from which file/s should be transferred"
  echo "Param: -u remote_user               -> User to connect to remote_host as"
  echo "Param: -d remote_dir                -> Fully qualified remote directory on remote_host"
  echo "Param: -f file_pattern              -> File pattern describing file/s to be transferred"
  echo "Param: -D local_dir                 -> Fully qualified directory on local host"
  echo "Param: -m mode                      -> Transfer mode: <get|put>. get -> files are retreived from remote_host | put -> files are sent to remote_host"
  return 0
}

function now() { # echos date and time now
  DATENOW=$(date +"%Y-%m-%d")
  TIMENOW=$(date +"%T")
  echo "$DATENOW $TIMENOW"
  return 0
}

function log() { # Log debug information
  if [ $1 -le $DEBUG_LEVEL -a $DEBUG_LEVEL -ne 0 ]; then
    echo "$(now): $THIS_SCRIPT: $2" | tee -a $LOGFILE
  fi
  return 0
}

################################################################################
# Initialise
DEBUG_LEVEL=2     # 0|1|2|3 -> None|Error|Info|All
LOGFILE="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/$(basename "${BASH_SOURCE[0]%.sh}.log")"
ERRORFILE="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/$(basename "${BASH_SOURCE[0]%.sh}.err")"
THIS_SCRIPT="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/$(basename "${BASH_SOURCE[0]}")";
THIS_SCRIPT_PATH="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)";
log 1 "################################################################################"
log 1 "# Initialise $THIS_SCRIPT"
log 2 "DEBUG_LEVEL: $DEBUG_LEVEL"
log 2 "LOGFILE: $LOGFILE"
log 2 "ERRORFILE: $ERRORFILE"
log 2 "THIS_SCRIPT: $THIS_SCRIPT"
log 2 "THIS_SCRIPT_PATH: $THIS_SCRIPT_PATH"

# Initialise expected params
NUM_RETRIES=
TIME_BETWEEN_RETRIES=
REMOTE_HOST=
REMOTE_USER=
REMOTE_DIR=
FILE_PATTERN=
LOCAL_DIR=
MODE=

# Accept command line arguments and process parameters
while getopts 'n:b:h:u:d:f:D:m:' Option
do
  case $Option in
    n) log 2 "-n default_number_of_retries option supplied was: $OPTARG"; NUM_RETRIES=$OPTARG ;;
    b) log 2 "-b time_between_retries option supplied was: $OPTARG";      TIME_BETWEEN_RETRIES=$OPTARG ;;
    h) log 2 "-h remote_host option supplied was: $OPTARG";               REMOTE_HOST=$OPTARG ;;
    u) log 2 "-u remote_user option supplied was: $OPTARG";               REMOTE_USER=$OPTARG ;;
    d) log 2 "-d remote_dir option supplied was: $OPTARG";                REMOTE_DIR=$OPTARG ;;
    f) log 2 "-f file_pattern option supplied was: $OPTARG";              FILE_PATTERN=$OPTARG ;;
    D) log 2 "-D local_dir option supplied was: $OPTARG";                 LOCAL_DIR=$OPTARG ;;
    m) log 2 "-m mode option supplied was: $OPTARG";                      MODE=$OPTARG ;;
    *) log 1 "Error: Invalid option supplied: $OPTARG"; usage; exit 2 ;;
  esac
done
shift $(($OPTIND - 1))

# Confirm mode is either put or get
if [ $MODE = 'put' ] || [ $MODE = 'get' ]; then
  log 3 "MODE: $MODE is valid"
else
  log 1 "Error. MODE: $MODE is invalid"; usage; exit 3
fi

# Transfer files
ATTEMPT_NUMBER=0;   log 3 "ATTEMPT_NUMBER: $ATTEMPT_NUMBER"
EXITSTATUS=-1;      log 3 "EXITSTATUS: $EXITSTATUS"
while [ $ATTEMPT_NUMBER -le $NUM_RETRIES ] && [ $EXITSTATUS -ne 0 ]; do

  let "ATTEMPT_NUMBER=ATTEMPT_NUMBER+1";            log 3 "ATTEMPT_NUMBER: $ATTEMPT_NUMBER"
  if [ $MODE = 'put' ]; then # put
    log 2 "Put attempt: $ATTEMPT_NUMBER"
    log 2 "Executing: sftp -b - $REMOTE_USER@$REMOTE_HOST >> $LOGFILE 2> $ERRORFILE"
    sftp -b - $REMOTE_USER@$REMOTE_HOST >> $LOGFILE 2> $ERRORFILE <<END_PUT
    lcd $LOCAL_DIR
    cd $REMOTE_DIR
    put $FILE_PATTERN $REMOTE_DIR
    quit
END_PUT
    log 2 "Put attempt: $ATTEMPT_NUMBER finished"
  else # get
    log 2 "Get attempt: $ATTEMPT_NUMBER"
    log 2 "Executing: sftp -b - $REMOTE_USER@$REMOTE_HOST >> $LOGFILE 2> $ERRORFILE"
    sftp -b - $REMOTE_USER@$REMOTE_HOST >> $LOGFILE 2> $ERRORFILE <<END_GET
    lcd $LOCAL_DIR
    cd $REMOTE_DIR
    get $REMOTE_DIR/$FILE_PATTERN $LOCAL_DIR
    quit
END_GET
    log 2 "Get attempt: $ATTEMPT_NUMBER finished"
  fi
  EXITSTATUS=`wc -m $ERRORFILE  | cut -d ' ' -f 1,1`;                    log 3 "EXITSTATUS: $EXITSTATUS"

  if [ "$EXITSTATUS" != "0" ]; then
    log 1 "Error: SFTP attempt number $ATTEMPT_NUMBER failed"
    log 1 "Error: Contents of $ERRORFILE follow:"
    log 1 "Error: ============== START OF ERROR FILE =============="
    log 1 "Error: `cat $ERRORFILE`"
    log 1 "Error: =============== END OF ERROR FILE ==============="
    sleep $TIME_BETWEEN_RETRIES
  fi

done

if [ "$EXITSTATUS" != "0" ]; then
  log 1 "Error: SFTP failed after $ATTEMPT_NUMBER attempts, exiting"
  log 1 "# Script $THIS_SCRIPT: Failed"
  log 1 "################################################################################"
  exit 4
else
  log 2 "SFTP attempt successful after $ATTEMPT_NUMBER attempts";
  log 1 "# Script $THIS_SCRIPT: Finished successfully"
  log 1 "################################################################################"
  exit 0
fi
